#!/usr/bin/env python3
from multiprocessing import Pool
from string import ascii_lowercase, ascii_uppercase
import fileinput
import re

REACT_SET = set(map(''.join, zip(ascii_lowercase, ascii_uppercase)))
REACT_SET = REACT_SET.union({r[::-1] for r in REACT_SET})
REACT_REGEX = re.compile(r'({})'.format('|'.join(REACT_SET)))


def react(polymer):
    count = 1
    while count > 0:
        polymer, count = REACT_REGEX.subn('', polymer)
    return len(polymer)


def react_without(polymer, letter):
    print('Reacting without letter {}'.format(letter))
    return react(polymer.replace(letter, '').replace(letter.upper(), ''))


def main():
    line = next(fileinput.input()).strip()

    print('Reacting full line')
    print(react(line))

    with Pool(4) as p:
        print(min(p.starmap(react_without, ((line, letter) for letter in ascii_lowercase))))


if __name__ == '__main__':
    main()
